package pipe.gui.widgets;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.io.IOException;
import java.util.List;
import java.util.regex.Pattern;

import javax.swing.*;

import dk.aau.cs.model.tapn.Colored.ColorType;
import dk.aau.cs.model.tapn.TimedArcPetriNet;
import org.jdesktop.swingx.JXComboBox;
import pipe.gui.CreateGui;
import dk.aau.cs.gui.undo.Command;
import dk.aau.cs.model.tapn.Constant;
import dk.aau.cs.model.tapn.TimedArcPetriNetNetwork;
import pipe.gui.Grid;

public class ColorTypeDialogPanel {
    private static final long serialVersionUID = 1L;
    private static final String toolTipColorComboBox = "Switch between the different defined color types";
    private static final String finiteEnumeration= "Finite Enumeration";
    private static final String cyclicEnumeration = "Cyclic Enumeration";
    private JRootPane rootPane;
    private TimedArcPetriNetNetwork network;
    private EscapableDialog dialog;
    private ColorType colorType;
    private String oldName;

    JPanel nameTextFieldPane;
    private JTextField nameTextField;
    Dimension size;
    private JLabel nameLabel;

    private JComboBox colorTypeComboBox;
    private JLabel colorTypeLabel;
    private JLabel elementTypeLabel;
    private JList jList;
    private ConstantsPane.ColorTypesListModel colorTypesListModel;
    private JScrollPane colorScroller;
    private JTextField colorTypeTextField;

    private JPanel container;
    private JPanel buttonContainer;
    JButton okButton;
    JButton cancelButton;


    public ColorTypeDialogPanel() throws IOException {
        initComponents();
    }

    public ColorTypeDialogPanel(JRootPane pane, TimedArcPetriNetNetwork network) throws IOException {
        rootPane = pane;
        this.network = network;
        initComponents();
        oldName = "";
        nameTextField.setText(oldName);

    }

    public ColorTypeDialogPanel(JRootPane pane, TimedArcPetriNetNetwork network, ColorType colortype) throws IOException {
        rootPane = pane;
        this.network = network;
        initComponents();
        this.colorType = colortype;
        oldName = colortype.getName();
        nameTextField.setText(oldName);


    }

    public void showDialog() {
        dialog = new EscapableDialog(CreateGui.getApp(),
                "Edit color type", true);
        dialog.add(container);
        dialog.getRootPane().setDefaultButton(okButton);
        dialog.setResizable(false);
        //dialog.setPreferredSize(new Dimension(500, 500));
        dialog.pack();
        dialog.setLocationRelativeTo(null);
        dialog.setVisible(true);

    }

    private void initComponents() throws IOException {
        container = new JPanel();
        buttonContainer = new JPanel();
        container.setLayout(new GridBagLayout());
        size = new Dimension(330, 30);
        GridBagConstraints gbc;

        createNameTextField();
        createNameLabel();
        createColorTypeTextField();
        createColorTypeLabel();
        createColorTypeComboBox();
        createElementTypeLabel();
        JPanel buttonPanel = createButtonPanel();

//

     //   okButton.addActionListener(new ActionListener() {
     //       public void actionPerformed(ActionEvent e) {
     //           onOK();
     //       }
     //   });
//
     //   cancelButton.addActionListener(new ActionListener() {
     //       public void actionPerformed(ActionEvent e) {
     //           exit();
     //       }
     //   });
//
        gbc = new GridBagConstraints();
        gbc.insets = new Insets(0, 8, 5, 8);
        gbc.gridx = 1;
        gbc.gridy = 3;
        gbc.gridwidth = 1;
        gbc.anchor = GridBagConstraints.EAST;
        container.add(buttonPanel,gbc);


    }

    private void createNameTextField() {
        nameTextField = new javax.swing.JTextField();
        nameTextField.setPreferredSize(size);
        nameTextField.addAncestorListener(new RequestFocusListener());
        nameTextField.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                okButton.requestFocusInWindow();
                okButton.doClick();
            }
        });

        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridx = 1;
        gbc.gridy = 0;
        gbc.gridwidth = 1;
        gbc.anchor = GridBagConstraints.WEST;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.insets = new Insets(4, 4, 2, 4);
        container.add(nameTextField, gbc);


    }

    private void createColorTypeTextField() {
        colorTypeTextField = new javax.swing.JTextField();
        colorTypeTextField.setPreferredSize(size);
        colorTypeTextField.addAncestorListener(new RequestFocusListener());
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridx = 1;
        gbc.gridy = 2;
        gbc.gridwidth = 1;
        gbc.anchor = GridBagConstraints.WEST;
        container.add(colorTypeTextField, gbc);


    }

    private void createColorTypeLabel() {
        colorTypeLabel = new javax.swing.JLabel();
        colorTypeLabel.setText("Color type: ");
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.insets = new Insets(2, 4, 2, 4);
        gbc.gridx = 0;
        gbc.gridy = 1;
        gbc.gridwidth = 1;
        gbc.anchor = GridBagConstraints.WEST;
        container.add(colorTypeLabel,gbc);
    }

    private void createNameLabel() {
        nameLabel = new JLabel();
        nameLabel.setText("Name: ");
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.gridwidth = 1;
        gbc.insets = new Insets(4, 4, 2, 4);
        gbc.anchor = GridBagConstraints.WEST;
        container.add(nameLabel, gbc);
    }

    private void createElementTypeLabel() {
        elementTypeLabel = new JLabel();
        elementTypeLabel.setText("Element type:");
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.insets = new Insets(2, 4, 2, 4);
        gbc.gridx = 0;
        gbc.gridy = 2;
        gbc.gridwidth = 1;
        gbc.anchor = GridBagConstraints.WEST;
        container.add(elementTypeLabel,gbc);
    }

    private void createColorTypeComboBox() {
        colorTypeComboBox = new JXComboBox(new String[]{finiteEnumeration, cyclicEnumeration});
        colorTypeComboBox.setToolTipText(toolTipColorComboBox);
        colorTypeComboBox.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                JComboBox source = (JXComboBox)e.getSource();
                String selectedString = source.getSelectedItem().toString();
                if(selectedString.equals(cyclicEnumeration)) {
                    jList.setModel(colorTypesListModel);
                    // Change the actions
                    okButton.setToolTipText("Det virker");
                }
                else if (selectedString.equals(finiteEnumeration)) {
                    jList.setModel((colorTypesListModel));
                    // Change the actions
                    okButton.setToolTipText("It's a workinga ");
                }
            }
        });

        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridx = 1;
        gbc.gridy = 1;
        gbc.gridwidth = 1;
        gbc.anchor = GridBagConstraints.EAST;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        container.add(colorTypeComboBox, gbc);
        colorScroller = new JScrollPane(jList);
    }

    private JPanel createButtonPanel() {
        JPanel buttonPanel = new JPanel();
        buttonPanel.setLayout(new GridBagLayout());

        okButton = new JButton("OK");
        okButton.setMaximumSize(new java.awt.Dimension(100, 25));
        okButton.setMinimumSize(new java.awt.Dimension(100, 25));
        okButton.setPreferredSize(new java.awt.Dimension(100, 25));

        okButton.setMnemonic(KeyEvent.VK_O);
        GridBagConstraints gbcOk = new GridBagConstraints();
        gbcOk.gridx = 1;
        gbcOk.gridy = 0;
        gbcOk.anchor = java.awt.GridBagConstraints.WEST;
        gbcOk.insets = new java.awt.Insets(5, 5, 5, 5);
        JButton cancelButton = new JButton("Cancel");
        cancelButton.setMaximumSize(new java.awt.Dimension(100, 25));
        cancelButton.setMinimumSize(new java.awt.Dimension(100, 25));
        cancelButton.setPreferredSize(new java.awt.Dimension(100, 25));

        cancelButton.setMnemonic(KeyEvent.VK_C);
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.gridwidth = java.awt.GridBagConstraints.RELATIVE;
        gbc.anchor = GridBagConstraints.EAST;

        cancelButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                exit();
            }
        });

        buttonPanel.add(cancelButton,gbc);
        buttonPanel.add(okButton,gbcOk);


        return buttonPanel;
    }

    private void createCancelButton() {
        cancelButton = new JButton();
        cancelButton.setText("Cancel");
        cancelButton.setMaximumSize(new java.awt.Dimension(100, 25));
        cancelButton.setMinimumSize(new java.awt.Dimension(100, 25));
        cancelButton.setPreferredSize(new java.awt.Dimension(100, 25));
        cancelButton.setMnemonic(KeyEvent.VK_C);
        cancelButton.setBounds(10,0,100,25);
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.gridheight = 3;
        gbc.gridwidth = java.awt.GridBagConstraints.RELATIVE;
        gbc.anchor = GridBagConstraints.SOUTH;
        buttonContainer.add(cancelButton, gbc);
    }

    private void createOKButton() {
        okButton = new JButton();
        okButton.setText("OK");
        okButton.setMaximumSize(new java.awt.Dimension(100, 25));
        okButton.setMinimumSize(new java.awt.Dimension(100, 25));
        okButton.setPreferredSize(new java.awt.Dimension(100, 25));
        okButton.setMnemonic(KeyEvent.VK_O);
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridx = 1;
        gbc.gridy = 0;
        gbc.gridheight = 3;
        gbc.anchor = GridBagConstraints.SOUTH;
        gbc.insets = new java.awt.Insets(5, 5, 5, 5);
        buttonContainer.add(okButton, gbc);
    }

    private void exit() {
        dialog.setVisible(false);
    }

    private void onOK() { }


}
