package pipe.gui.widgets;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.*;

import java.io.IOException;


import javax.swing.*;
import javax.swing.border.TitledBorder;
import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import dk.aau.cs.model.tapn.Colored.ColorType;
import dk.aau.cs.model.tapn.Colored.Variable;
import dk.aau.cs.util.Require;
import org.jdesktop.swingx.JXComboBox;
import pipe.gui.CreateGui;
import dk.aau.cs.gui.TabContent;
import dk.aau.cs.gui.undo.Command;
import dk.aau.cs.gui.undo.SortConstantsCommand;
import dk.aau.cs.model.tapn.Constant;
import dk.aau.cs.model.tapn.TimedArcPetriNetNetwork;
import dk.aau.cs.gui.components.ConstantsListModel;
import dk.aau.cs.gui.components.NonsearchableJList;

public class ConstantsPane extends JPanel {
	private static final long serialVersionUID = -7883351020889779067L;
	private JPanel constantsPanel;
	private JScrollPane constantsScroller;
	private JPanel buttonsPanel;

	private ColorTypesListModel colorTypesListModel;
	private VariablesListModel variablesListModel;
	private GlobalConstantsListModel globalConstantsListModel;
	private JXComboBox constantsColorTypesVariablesComboBox;
	private JList list;
	private ConstantsListModel listModel;
	private JButton editBtn;
	private JButton removeBtn;

	private TabContent parent;
	private JButton moveUpButton;
	private JButton moveDownButton;
	private JButton sortButton;

	private static final String titleBorder = "Global constants, Color types and Variables";
    private static final String titleBorderToolTip = "Declaration of global constants that can be used in intervals and age invariants, declaration of Colors types and declaration of Variables";

	private static final String CONSTANTS = "Constants";
	private static final String COLORTYPES = "Color type";
	private static final String VARIABLES = "Variables";

	private static final String toolTipComboBox = "Switch between constants, colors and variables";
	private static final String toolTipEditConstant = "Edit the value of the selected constant";
	private static final String toolTipRemoveConstant = "Remove the selected constant";
	private static final String toolTipNewConstant = "Create a new constant";
	private static final String toolTipSortConstants = "Sort the constants alphabetically";
	private final static String toolTipMoveUp = "Move the selected constant up";
	private final static String toolTipMoveDown = "Move the selected constant down";
	//private static final String toolTipGlobalConstantsLabel = "Here you can define a global constant for reuse in different places.";


	public ConstantsPane(boolean enableAddButton, TabContent currentTab) {
		parent = currentTab;

        list = new NonsearchableJList();
		constantsPanel = new JPanel(new GridBagLayout());
		buttonsPanel = new JPanel(new GridBagLayout());

		globalConstantsListModel = new GlobalConstantsListModel(currentTab.network());
		colorTypesListModel = new ColorTypesListModel(currentTab.network());
		variablesListModel = new VariablesListModel(currentTab.network());


		listModel = new ConstantsListModel(parent.network());
		listModel.addListDataListener(new ListDataListener() {
			public void contentsChanged(ListDataEvent arg0) {				
			}

			public void intervalAdded(ListDataEvent arg0) {
				list.setSelectedIndex(arg0.getIndex0());
				list.ensureIndexIsVisible(arg0.getIndex0());
			}

			public void intervalRemoved(ListDataEvent arg0) {
				int index = (arg0.getIndex0() == 0) ? 0 : (arg0.getIndex0() - 1);
				list.setSelectedIndex(index);
				list.ensureIndexIsVisible(index);
			}
			
		});
        /*
		list = new NonsearchableJList(listModel);
		list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		*/
        /*
		list.addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent e) {
				if (!(e.getValueIsAdjusting())) {
					if (list.getSelectedIndex() == -1) {
						editBtn.setEnabled(false);
						removeBtn.setEnabled(false);
					} else {
						removeBtn.setEnabled(true);
						editBtn.setEnabled(true);						
					}

					int index = list.getSelectedIndex();
					if(index > 0)
						moveUpButton.setEnabled(true);
					else
						moveUpButton.setEnabled(false);


					if(index < parent.network().constants().size() - 1)
						moveDownButton.setEnabled(true);
					else
						moveDownButton.setEnabled(false);
				}
			}
		});
        */

        /* TODO: Has to be modified such that it incorporates all three different listmodels instead of only constants.
		list.addKeyListener(new KeyAdapter() {
			public void keyPressed(KeyEvent arg0) {
				ListModel model = list.getModel();
				if (model.getSize()>0) {
					Constant c = (Constant) model.getElementAt(list.getSelectedIndex());
					if (c != null) {
						if (arg0.getKeyCode() == KeyEvent.VK_LEFT) {
							if (!(c.lowerBound() == c.value())){
								Command edit = parent.network().updateConstant(c.name(), new Constant(
										c.name(), c.value()-1));
								CreateGui.getDrawingSurface().getUndoManager().addNewEdit(edit);
								parent.network().buildConstraints();
							}
						}
						else if (arg0.getKeyCode() == KeyEvent.VK_RIGHT) {
							if (!(c.upperBound() == c.value())){
								Command edit = parent.network().updateConstant(c.name(), new Constant(
										c.name(), c.value()+1));
								CreateGui.getDrawingSurface().getUndoManager().addNewEdit(edit);
								parent.network().buildConstraints();
							}
						}
						else if (arg0.getKeyCode() == KeyEvent.VK_UP) {
							if(list.getSelectedIndex() > 0){
								highlightConstant(list.getSelectedIndex()-1);
							}
						}
						else if (arg0.getKeyCode() == KeyEvent.VK_DOWN) {
							if(list.getSelectedIndex() < list.getModel().getSize()-1){
								highlightConstant(list.getSelectedIndex()+1);
							}
						}
					}
				}
			}
		});

		*/
		addConstantsComponents();
		addConstantsButtons(enableAddButton);
		
		list.addFocusListener(new FocusAdapter() {
			@Override
			public void focusLost(FocusEvent arg0) {
				removeConstantHighlights();
			}
		});

		setLayout(new BorderLayout());
		this.add(constantsPanel, BorderLayout.CENTER);
		this.add(buttonsPanel, BorderLayout.PAGE_END);

		setBorder(BorderFactory.createCompoundBorder(
				BorderFactory.createTitledBorder(titleBorder),
				BorderFactory.createEmptyBorder(3, 3, 3, 3))
				);
		this.setToolTipText(titleBorderToolTip);

		//this.setToolTipText(toolTipGlobalConstantsLabel);
		showConstants();


		this.addComponentListener(new ComponentListener() {
			int minimumHegiht = ConstantsPane.this.getMinimumSize().height;
			public void componentShown(ComponentEvent e) {
			}
			
			
			public void componentResized(ComponentEvent e) {
				if(ConstantsPane.this.getSize().height <= minimumHegiht){
					sortButton.setVisible(false);
				} else {
					sortButton.setVisible(true);
				}
			}
			
			
			public void componentMoved(ComponentEvent e) {
			}
			
			
			public void componentHidden(ComponentEvent e) {
			}
		});
		
		this.setMinimumSize(new Dimension(this.getMinimumSize().width, this.getMinimumSize().height - sortButton.getMinimumSize().height));
    }
	
	private void highlightConstant(int index){
		ListModel model = list.getModel();
		Constant c = (Constant) model.getElementAt(index);
		
		if(c != null && !c.hasFocus()){
			for(int i = 0; i < model.getSize(); i++){
				((Constant) model.getElementAt(i)).setFocused(false);
			}
			c.setFocused(true);
			CreateGui.getCurrentTab().drawingSurface().repaintAll();
		}
	}
	
	public void removeConstantHighlights(){
		ListModel model = list.getModel();
		for(int i = 0; i < model.getSize(); i++){
			((Constant) model.getElementAt(i)).setFocused(false);
		}
		try{
			CreateGui.getCurrentTab().drawingSurface().repaintAll();
		}catch(Exception e){
			// It is okay, the tab has just been closed
		}
	}

	private void addConstantsButtons(boolean enableAddButton) {
		editBtn = new JButton("Edit");
		editBtn.setEnabled(false);
		editBtn.setToolTipText(toolTipEditConstant);
		editBtn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				Constant c = (Constant) list.getSelectedValue();
				showEditConstantDialog(c, list.getSelectedIndex());
			}
		});
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.anchor = GridBagConstraints.WEST;
		buttonsPanel.add(editBtn, gbc);


		removeBtn = new JButton("Remove");
		removeBtn.setEnabled(false);
		removeBtn.setToolTipText(toolTipRemoveConstant);
		removeBtn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				String constName = ((Constant) list.getSelectedValue()).name();
				removeConstant(constName);
			}
		});
		gbc = new GridBagConstraints();
		gbc.gridx = 1;
		gbc.anchor = GridBagConstraints.WEST;
		buttonsPanel.add(removeBtn, gbc);

		JButton addConstantButton = new JButton("New");
		addConstantButton.setToolTipText(toolTipNewConstant);
		addConstantButton.setEnabled(enableAddButton);
		addConstantButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
			    String source = constantsColorTypesVariablesComboBox.getSelectedItem().toString();
			    if (source.equals(CONSTANTS)) {
                    showEditConstantDialog(null);
                }
                else if (source.equals(VARIABLES)) {
                    showEditVariableDialog(null);
                }
                else if (source.equals(COLORTYPES)) {
                    showEditColorTypeDialog(null);
                }

			}
		});
		gbc = new GridBagConstraints();
		gbc.gridx = 2;
		gbc.anchor = GridBagConstraints.WEST;
		buttonsPanel.add(addConstantButton, gbc);
	}

	public void showConstants() {
		TimedArcPetriNetNetwork model = parent.network();
		if (model == null)
			return;

		listModel.updateAll();

	}

	private void addConstantsComponents() {
        list.addListSelectionListener(new ListSelectionListener() {
            @Override
            public void valueChanged(ListSelectionEvent e) {
                if (!e.getValueIsAdjusting()) {
                    JList source = (JList)e.getSource();
                    if (source.getSelectedIndex() == -1) {
                        removeBtn.setEnabled(false);
                        editBtn.setEnabled(false);
                    }
                    else  {
                        removeBtn.setEnabled(true);
                        editBtn.setEnabled(true);
                    }

                    int index = list.getSelectedIndex();
                    if (index > 0) {
                        moveUpButton.setEnabled(true);
                    }
                    else {
                        moveUpButton.setEnabled(false);
                    }

                    if (isDisplayingColorTypes()) {
                        if (index < colorTypesListModel.getSize() - 1) {
                            moveDownButton.setEnabled(true);
                        }
                        else {
                            moveDownButton.setEnabled(false);
                        }
                    }
                    else if(isDisplayingVariables()) {
                        if (index < variablesListModel.getSize() -1) {
                            moveDownButton.setEnabled(true);
                        }
                        else {
                            moveDownButton.setEnabled(false);
                        }
                    }
                    else {
                        if (index < globalConstantsListModel.getSize() - 1) {
                            moveDownButton.setEnabled(true);
                        }
                        else {
                            moveDownButton.setEnabled(false);
                        }
                    }
                }

            }
        });

        list.addMouseListener(createDoubleClickMouseAdapter());

        constantsColorTypesVariablesComboBox = new JXComboBox(new String[]{CONSTANTS, COLORTYPES, VARIABLES });
        constantsColorTypesVariablesComboBox.setToolTipText(toolTipComboBox);
        constantsColorTypesVariablesComboBox.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                JComboBox source = (JXComboBox)e.getSource();
                String selectedItem = source.getSelectedItem().toString();
                if (selectedItem.equals(CONSTANTS)) {
                    list.setModel(listModel);
                    editBtn.setToolTipText(toolTipEditConstant);
                    removeBtn.setToolTipText(toolTipRemoveConstant);
                }
                else if (selectedItem.equals(COLORTYPES)) {
                    list.setModel(colorTypesListModel);
                    editBtn.setToolTipText("just the tip");
                }
                else if (selectedItem.equals(VARIABLES)) {
                    list.setModel(variablesListModel);
                    editBtn.setToolTipText("just the tip twice");
                }
            }
        });
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.anchor = GridBagConstraints.NORTH;
        gbc.fill = GridBagConstraints.HORIZONTAL;

        constantsPanel.add(constantsColorTypesVariablesComboBox, gbc);


		constantsScroller = new JScrollPane(list);

		gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.gridheight = 3;
		gbc.weightx = 1;
		gbc.weighty = 1;
		gbc.fill = GridBagConstraints.BOTH;
		gbc.anchor = GridBagConstraints.NORTHWEST;
		constantsPanel.add(constantsScroller, gbc);

		moveUpButton = new JButton(new ImageIcon(Thread.currentThread().getContextClassLoader().getResource("resources/Images/Up.png")));
		moveUpButton.setEnabled(false);
		moveUpButton.setToolTipText(toolTipMoveUp);
		moveUpButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				int index = list.getSelectedIndex();

				if(index > 0) {
					parent.swapConstants(index, index-1);
					showConstants();
					list.setSelectedIndex(index-1);
				}
			}
		});

		gbc = new GridBagConstraints();
		gbc.gridx = 1;
		gbc.gridy = 0;
		gbc.anchor = GridBagConstraints.SOUTH;
		constantsPanel.add(moveUpButton,gbc);

		moveDownButton = new JButton(new ImageIcon(Thread.currentThread().getContextClassLoader().getResource("resources/Images/Down.png")));
		moveDownButton.setEnabled(false);
		moveDownButton.setToolTipText(toolTipMoveDown);
		moveDownButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				int index = list.getSelectedIndex();

				if(index < parent.network().constants().size() - 1) {
					parent.swapConstants(index, index+1);
					showConstants();
					list.setSelectedIndex(index+1);
				}
			}
		});

		gbc = new GridBagConstraints();
		gbc.gridx = 1;
		gbc.gridy = 1;
		gbc.anchor = GridBagConstraints.NORTH;
		constantsPanel.add(moveDownButton,gbc);

		//Sort button
		sortButton = new JButton(new ImageIcon(Thread.currentThread().getContextClassLoader().getResource("resources/Images/Sort.png")));
		sortButton.setToolTipText(toolTipSortConstants);
		sortButton.setEnabled(true);
		sortButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				Command sortConstantsCommand = new SortConstantsCommand(parent, ConstantsPane.this);
				CreateGui.getDrawingSurface().getUndoManager().addNewEdit(sortConstantsCommand);
				sortConstantsCommand.redo();
			}
		});

		gbc = new GridBagConstraints();
		gbc.gridx = 1;
		gbc.gridy = 2;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.anchor = GridBagConstraints.NORTH;
		constantsPanel.add(sortButton,gbc);
	}

	private void showEditColorTypeDialog(ColorType colorType) {
		ColorTypeDialogPanel panel = null;
		if (colorType != null) {
			try {
				panel = new ColorTypeDialogPanel(new JRootPane(), parent.network(), colorType);
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		else {
			try {
				panel = new ColorTypeDialogPanel(new JRootPane(), parent.network());
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		panel.showDialog();
    }

    private void showEditVariableDialog(Variable variable) {
	    VariablesDialogPanel panel = null;
        if (variable != null) {
            try {
                panel = new VariablesDialogPanel(new JRootPane(), variablesListModel, parent.network(), variable);
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        else {
            try {
                panel = new VariablesDialogPanel(new JRootPane(), parent.network());
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        panel.showDialog();
        variablesListModel.updateName();
    }

	private void showEditConstantDialog(Constant constant) {	
		ConstantsDialogPanel panel = null;
		if (constant != null)
			try {
				panel = new ConstantsDialogPanel(new JRootPane(),
						parent.network(), constant);
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		else
			try {
				panel = new ConstantsDialogPanel(new JRootPane(),
						parent.network());
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		panel.showDialog();
		showConstants();
	}

	private void showEditConstantDialog(Constant constant, int selectedIndex) {	
		ConstantsDialogPanel panel = null;
		if (constant != null)
			try {
				panel = new ConstantsDialogPanel(new JRootPane(),
						parent.network(), constant);
			} catch (IOException e) {
				e.printStackTrace();
			}
		else
			try {
				panel = new ConstantsDialogPanel(new JRootPane(),
						parent.network());
			} catch (IOException e) {
				e.printStackTrace();
			}
		panel.showDialog();
		showConstants();
	}

	protected void removeConstant(String name) {
		TimedArcPetriNetNetwork model = parent.network();
		Command edit = model.removeConstant(name);
		if (edit == null) {
			JOptionPane.showMessageDialog(CreateGui.getApp(),
					"You cannot remove a constant that is used in the net.\nRemove all references "
							+ "to the constant in the net and try again.",
							"Constant in use", JOptionPane.ERROR_MESSAGE);
		} else
			parent.drawingSurface().getUndoManager().addNewEdit(edit);

		//showConstants();
	}

	public void setNetwork(TimedArcPetriNetNetwork tapnNetwork) {
		listModel.setNetwork(tapnNetwork);
	}

	public void selectFirst() {
		list.setSelectedIndex(0);

	}

	private MouseListener createDoubleClickMouseAdapter() {
	    return new MouseAdapter() {
	        @Override
            public void mouseClicked(MouseEvent arg0) {
	            if (!list.isSelectionEmpty()) {
	                if (arg0.getButton() == MouseEvent.BUTTON1 && arg0.getClickCount() == 2) {
	                    if (isDisplayingColorTypes()) {
                            showEditConstantDialog((Constant)list.getSelectedValue());
                        }
                        else if (isDisplayingGlobalConstants()) {
                            showEditConstantDialog((Constant)list.getSelectedValue());
                        }
                        else {
                            showEditConstantDialog((Constant)list.getSelectedValue());
                        }
                    }
                }
            }
        };
    }

	protected boolean isDisplayingGlobalConstants() {
	    return constantsColorTypesVariablesComboBox.getSelectedItem().equals(CONSTANTS);
    }

    protected boolean isDisplayingColorTypes() {
	    return constantsColorTypesVariablesComboBox.getSelectedItem().equals(COLORTYPES);
    }

    protected boolean isDisplayingVariables() {
	    return constantsColorTypesVariablesComboBox.getSelectedItem().equals(VARIABLES);
    }

	public class GlobalConstantsListModel extends AbstractListModel {
        private static final long serialVersionUID = 1L;
        private TimedArcPetriNetNetwork network;

        public GlobalConstantsListModel(TimedArcPetriNetNetwork network) {
            this.network = network;

            addListDataListener(new ListDataListener() {
                public void intervalAdded(ListDataEvent arg0) {
                    list.setSelectedIndex(arg0.getIndex0());
                    list.ensureIndexIsVisible(arg0.getIndex0());
                }

                public void intervalRemoved(ListDataEvent arg0) {
                    int index = (arg0.getIndex0() == 0) ? 0 : (arg0.getIndex0() - 1);
                    list.setSelectedIndex(index);
                    list.ensureIndexIsVisible(index);
                }

                public void contentsChanged(ListDataEvent e) {
                }
            });
        }

        public int getSize() {
            return network.constants().size();
        }

        public Object getElementAt(int i) {
            return network.getConstant(i);
        }

        public void swap(int currentIndex, int newIndex) {
            network.swapConstants(currentIndex, newIndex);
        }

        public Constant[] sort(){
            Constant[] oldOrder = network.sortConstants();
            fireContentsChanged(this, 0, getSize());
            return oldOrder;
        }

        public void undoSort(Constant[] oldOrder) {
            network.undoSort(oldOrder);
            fireContentsChanged(this, 0, getSize());
        }

        public void addElement(String name, int value) {
            network.addConstant(name, value);
            fireIntervalAdded(this, getSize()-1, getSize());
        }

        public void removeElement(String constantName) {
            network.removeConstant(constantName);
            fireContentsChanged(this, 0, getSize());
            int numElements = list.getModel().getSize();
            if(numElements <= 1) {
                moveDownButton.setEnabled(false);
                moveUpButton.setEnabled(false);
            }
            if (numElements <= 0) {
                editBtn.setEnabled(false);
                editBtn.setEnabled(false);
            }
        }

        public void updateName() {
            fireContentsChanged(this, 0, getSize());
        }

        public void setNetwork(TimedArcPetriNetNetwork network) {
            Require.that(network != null, "network cannot be null");
            this.network = network;
            fireContentsChanged(this, 0, getSize());
        }
    }

    public class ColorTypesListModel extends AbstractListModel {
        private static final long serialVersionUID = 1L;
        private TimedArcPetriNetNetwork network;

        public ColorTypesListModel(TimedArcPetriNetNetwork network) {
            Require.that(network != null, "network must not be null");
            this.network = network;

            addListDataListener(new ListDataListener() {
                public void intervalAdded(ListDataEvent arg0) {
                    list.setSelectedIndex(arg0.getIndex0());
                    list.ensureIndexIsVisible(arg0.getIndex0());
                }

                public void intervalRemoved(ListDataEvent arg0) {
                    int index = (arg0.getIndex0() == 0) ? 0 : (arg0.getIndex0() - 1);
                    list.setSelectedIndex(index);
                    list.ensureIndexIsVisible(index);
                }

                public void contentsChanged(ListDataEvent e) {
                }
            });
        }

        public void swap(int currentIndex, int newIndex) {
            network.swapColorTypes(currentIndex, newIndex);
        }

        public ColorType[] sort() {
            ColorType[] oldOrder = network.sortColorTypes();
            fireContentsChanged(this, 0, getSize());
            return oldOrder;
        }

        public void undoSort(ColorType[] oldOrder) {
            network.undoSort(oldOrder);
        }

        public int getSize() {
            return network.numberOfColorTypes();
        }

        public Object getElementAt(int i) {
            return network.getColorTypeByIndex(i);
        }

        public void addElement(ColorType colorType) {
            network.add(colorType);
            fireIntervalAdded(this, network.numberOfColorTypes()-1, network.numberOfColorTypes());
        }

        public void removeElement(ColorType colorType) {
            network.remove(colorType);
            fireContentsChanged(this, 0, getSize());
            int numElements = list.getModel().getSize();
            if (numElements <= 1) {
                moveDownButton.setEnabled(false);
                moveUpButton.setEnabled(false);
            }
            if (numElements <= 0) {
                removeBtn.setEnabled(false);
                editBtn.setEnabled(false);
            }
        }

        public void updateName() {
            fireContentsChanged(this, 0, getSize());
        }

        public void setNetwork(TimedArcPetriNetNetwork network) {
            Require.that(network != null, "network cannot be null");
            this.network = network;
            fireContentsChanged(this, 0, network.numberOfColorTypes());
        }
    }

    public class VariablesListModel extends AbstractListModel {
        private static final long serialVersionUID = 1L;
        private TimedArcPetriNetNetwork network;

        public VariablesListModel(TimedArcPetriNetNetwork network) {
            this.network = network;

            addListDataListener(new ListDataListener() {
                public void intervalAdded(ListDataEvent arg0) {
                    list.setSelectedIndex(arg0.getIndex0());
                    list.ensureIndexIsVisible(arg0.getIndex0());
                }

                public void intervalRemoved(ListDataEvent arg0) {
                    int index = (arg0.getIndex0() == 0) ? 0 : (arg0.getIndex0() - 1);
                    list.setSelectedIndex(index);
                    list.ensureIndexIsVisible(index);
                }

                public void contentsChanged(ListDataEvent e) {
                }
            });
        }

        public Object getElementAt(int index) {
            return network.getVariableByIndex(index);
        }

        public void swap(int currentIndex, int newIndex) {
            network.swapConstants(currentIndex, newIndex);
        }

        public Variable[] sort() {
            Variable[] oldOrder = network.sortVariables();
            fireContentsChanged(this, 0, getSize());
            return oldOrder;
        }

        public void undoSort(Variable[] oldOrder) {
            network.undoSort(oldOrder);
            fireContentsChanged(this, 0, getSize());
        }

        public int getSize() {
            return network.numberOfVariables();
        }

        public void addElement(Variable variable) {
            network.add(variable);
            fireIntervalAdded(this, network.numberOfVariables()-1, network.numberOfVariables());
        }

        public void removeElement(Variable variable) {
            network.remove(variable);
            fireContentsChanged(this, 0, getSize());
            int numElements = list.getModel().getSize();
            if(numElements <= 1) {
                moveDownButton.setEnabled(false);
                moveUpButton.setEnabled(false);
            }
            if (numElements <= 0) {
                editBtn.setEnabled(false);
                editBtn.setEnabled(false);
            }
        }

        public void updateName() {
            fireContentsChanged(this, 0, getSize());
        }

        public void setNetwork(TimedArcPetriNetNetwork network) {
            Require.that(network != null, "network cannot be null");
            this.network = network;
            fireContentsChanged(this, 0, network.numberOfVariables());
        }
    }

}
